/** @file		NetF_AvgRate.h
 *  @brief	Accurate average data rate calculation
 *  @author	netfinity
 */
#pragma once

#include "NetF/NetF_MilliSeconds.h"
#include "NetF/NetF_Tick.h"

/**
 *  @brief	Template class for calculating flow/data rates
 */
template<class _Ta, class _Td>
class CAvgRate
{
private:
	CTick				m_tickLastRecalc;
	CMilliSeconds 		m_msSamplePeriod;
	unsigned int 		m_nSamples;
	_Td					m_tPendingMeasure;
	_Ta					m_tLastEstAverage;
	double				m_fSumSamples;
	struct Sample
	{
		double			m_fSample;
		CTick			m_tickSample;
	};
	CList<Sample*>		m_listSamples;
public:
	CAvgRate(const unsigned int Samples = 20, const CMilliSeconds& SamplePeriod = CMilliSeconds(500))
	{
		m_tickLastRecalc = CTick::GetCurrentTick();
		m_msSamplePeriod = SamplePeriod;
		m_nSamples = Samples;
		m_tPendingMeasure = _Td(0);
		m_tLastEstAverage = _Ta(0);
		m_fSumSamples = 0;
	}
	~CAvgRate()
	{
		while (m_listSamples.GetCount() > 0)
		{
			Sample*	pSample = m_listSamples.RemoveHead();
			delete pSample;
		}
	}
	// Estimate (predict) the current rate using calculated values
	_Ta GetCurrentRate() const {return m_tLastEstAverage;}
	// Type cast and arithmetic operators
	operator _Ta () const {return GetCurrentRate();}
	bool operator ==(_Ta value) const {return GetCurrentRate() == value;}
	bool operator !=(_Ta value) const {return GetCurrentRate() != value;}
	bool operator >=(_Ta value) const {return GetCurrentRate() >= value;}
	bool operator <=(_Ta value) const {return GetCurrentRate() <= value;}
	bool operator >(_Ta value) const {return GetCurrentRate() > value;}
	bool operator <(_Ta value) const {return GetCurrentRate() < value;}
	_Ta operator +(_Ta value) const {return GetCurrentRate() + value;}
	_Ta operator -(_Ta value) const {return GetCurrentRate() - value;}
	// Set calculation parameters
	void SetSamples(const unsigned int Samples) {m_nSamples = Samples; Reset();}
	void SetSamplePeriod(const CMilliSeconds& SamplePeriod) {m_msSamplePeriod = SamplePeriod; Reset();}
	// Reset rate calculation
	void Reset()
	{
		m_tickLastRecalc = CTick::GetCurrentTick();
		m_tPendingMeasure = _Td(0);
		m_tLastEstAverage = _Ta(0);
		m_fSumSamples = 0;
		while (m_listSamples.GetCount() > 0)
		{
			Sample*	pSample = m_listSamples.RemoveHead();
			delete pSample;
		}
	}
	// Update rate calculation with new measure
	void Update(const _Td& Measure)
	{
		CTick	tickCurrent = CTick::GetCurrentTick();
		m_tPendingMeasure += Measure;
		if (m_tickLastRecalc.Elapsed(tickCurrent) >= m_msSamplePeriod)
		{
			/*double	a, b, N, D, x, y;
			double	sum_x = 0, sum_y = 0, sum_xy = 0, sum_x2 = 0;
			double	avg_half = 0, avg_full = 0;*/
			double	N, secSamplePeriod = (double) m_msSamplePeriod.GetValue() / 1000.0f;
			unsigned int i;
//			POSITION pos;
			// Add a new sample(s) to tail of sample list
			double			fCurDatarate = (double) m_tPendingMeasure.GetValue() * 1000.0f / (double) m_tickLastRecalc.Elapsed(tickCurrent).GetValue();
			unsigned int	nSamplesToAdd = m_tickLastRecalc.Elapsed(tickCurrent) / m_msSamplePeriod;
			if (nSamplesToAdd > m_nSamples)
			{
				m_tickLastRecalc = m_tickLastRecalc + m_msSamplePeriod * (nSamplesToAdd - m_nSamples);
				nSamplesToAdd = m_nSamples;
			}
			m_tPendingMeasure -= _Td(fCurDatarate * nSamplesToAdd * secSamplePeriod);
			for (i = 0; i < min(nSamplesToAdd, m_nSamples); ++i)
			{
				Sample*	pNewSample = new Sample;
				pNewSample->m_fSample = fCurDatarate ;
				pNewSample->m_tickSample = m_tickLastRecalc + m_msSamplePeriod;
				m_listSamples.AddHead(pNewSample);
				m_tickLastRecalc = pNewSample->m_tickSample;
				m_fSumSamples += fCurDatarate;
			}
			// Trim sample list
			while ((unsigned int) m_listSamples.GetCount() > m_nSamples)
			{
				Sample* pSample = m_listSamples.RemoveTail();
				m_fSumSamples -= pSample->m_fSample;
				delete pSample;
			}
			// Process sample list and sum measures together
			// I could have done this an other way, but then there would have been problems when tick counter wraps around
			/*i = 0;
			for (pos = m_listSamples.GetHeadPosition(); pos != 0;)
			{
				Sample*	pSample = m_listSamples.GetNext(pos);
				x = (double) pSample->m_tickSample.Elapsed(tickCurrent).GetValue();
				y = pSample->m_fMeasure;
				sum_x += x;
				sum_y += y;
				sum_xy += x * y;
				sum_x2 += x * x;
				avg_full += y;
				if (i < m_nSamples / 2)
					avg_half += y;
				++i;
			}
			avg_half /= (double) (m_nSamples / 2);
			avg_full /= (double) m_nSamples;*/
			// Calculate average: fit samples to the function y=ax+b
			N = m_listSamples.GetCount();
			if (N >= 1)
			{
				/*D = N * sum_x2 - sum_x * sum_x;
				a = (N * sum_xy - sum_x * sum_y) / D;
				b = (sum_y - a * sum_x) / N;
				x = (double) (m_nSamples / 4) * secSamplePeriod; // One quarter
				y = a * x + b;
				if (avg_half > (double) m_tLastEstAverage.GetValue())
					m_tLastEstAverage = _Ta(min(avg_half, y));
				else if (avg_half < (double) m_tLastEstAverage.GetValue())
					m_tLastEstAverage = _Ta(max(avg_half, y));*/
				m_tLastEstAverage = _Ta(m_fSumSamples / N);					
			}
			else
				m_tLastEstAverage = _Ta(0);
		}
	}
};
