/** @file		NetF_BytesPerSecond.h
 *  @brief	Type safe data rate calculations
 *  @author	netfinity
 */
#pragma once

#include <cfloat>

/**
 *  @brief	Container for data rates
 */
class CBytesPerSecond
{
public:
	CBytesPerSecond(const float fBytesPerSecond = 0.0);
	CBytesPerSecond(const CBytesPerSecond& bytespersecond);
	// Conversions
	float GetValue() const;
	// Special values
	static CBytesPerSecond Infinite() {return CBytesPerSecond(FLT_MAX);}
	// Assignments
	CBytesPerSecond operator += (const CBytesPerSecond& bytespersecond);
	CBytesPerSecond operator -= (const CBytesPerSecond& bytespersecond);
	// Comparisations
	bool operator == (const CBytesPerSecond& bytespersecond) const;
	bool operator != (const CBytesPerSecond& bytespersecond) const;
	bool operator >= (const CBytesPerSecond& bytespersecond) const;
	bool operator <= (const CBytesPerSecond& bytespersecond) const;
	bool operator > (const CBytesPerSecond& bytespersecond) const;
	bool operator < (const CBytesPerSecond& bytespersecond) const;
	// Arithmetics
	CBytesPerSecond operator - (void) const;
	CBytesPerSecond operator + (const CBytesPerSecond& bytespersecond) const;
	CBytesPerSecond operator - (const CBytesPerSecond& bytespersecond) const;
	CBytesPerSecond operator * (const float& multiplier) const;
	CBytesPerSecond operator / (const float& divisor) const;
	CBytesPerSecond operator % (const CBytesPerSecond& bytespersecond) const;
private:
	float	m_fBytesPerSecond;
};

//
//	inline operations
//

#include "NetF/NetF_MilliSeconds.h"
#include "NetF/NetF_Bytes.h"

inline
CBytesPerSecond::CBytesPerSecond(const float fBytesPerSecond)
{
	m_fBytesPerSecond = fBytesPerSecond;
}

inline
CBytesPerSecond::CBytesPerSecond(const CBytesPerSecond& bytespersecond)
{
	m_fBytesPerSecond = bytespersecond.m_fBytesPerSecond;
}

inline
float
CBytesPerSecond::GetValue() const
{
	return m_fBytesPerSecond;
}

inline
bool
CBytesPerSecond::operator == (const CBytesPerSecond& bytespersecond) const
{
	return m_fBytesPerSecond == bytespersecond.m_fBytesPerSecond;
}

inline
bool
CBytesPerSecond::operator != (const CBytesPerSecond& bytespersecond) const
{
	return m_fBytesPerSecond != bytespersecond.m_fBytesPerSecond;
}

inline
bool
CBytesPerSecond::operator >= (const CBytesPerSecond& bytespersecond) const
{
	return m_fBytesPerSecond >= bytespersecond.m_fBytesPerSecond;
}

inline
bool
CBytesPerSecond::operator <= (const CBytesPerSecond& bytespersecond) const
{
	return m_fBytesPerSecond <= bytespersecond.m_fBytesPerSecond;
}

inline
bool
CBytesPerSecond::operator > (const CBytesPerSecond& bytespersecond) const
{
	return m_fBytesPerSecond > bytespersecond.m_fBytesPerSecond;
}

inline
bool
CBytesPerSecond::operator < (const CBytesPerSecond& bytespersecond) const
{
	return m_fBytesPerSecond < bytespersecond.m_fBytesPerSecond;
}

inline
CBytesPerSecond
CBytesPerSecond::operator += (const CBytesPerSecond& bytespersecond)
{
	return m_fBytesPerSecond += bytespersecond.m_fBytesPerSecond;
}

inline
CBytesPerSecond
CBytesPerSecond::operator -= (const CBytesPerSecond& bytespersecond)
{
	return m_fBytesPerSecond -= bytespersecond.m_fBytesPerSecond;
}

inline
CBytesPerSecond
CBytesPerSecond::operator - (void) const
{
	return CBytesPerSecond(-m_fBytesPerSecond);
}

inline
CBytesPerSecond
CBytesPerSecond::operator + (const CBytesPerSecond& bytespersecond) const
{
	return CBytesPerSecond(m_fBytesPerSecond + bytespersecond.m_fBytesPerSecond);
}

inline
CBytesPerSecond
CBytesPerSecond::operator - (const CBytesPerSecond& bytespersecond) const
{
	return CBytesPerSecond(m_fBytesPerSecond - bytespersecond.m_fBytesPerSecond);
}

inline
CBytesPerSecond
CBytesPerSecond::operator * (const float& multiplier) const
{
	return CBytesPerSecond(m_fBytesPerSecond * multiplier);
}

inline
CBytesPerSecond
CBytesPerSecond::operator / (const float& divisor) const
{
	return CBytesPerSecond(m_fBytesPerSecond / divisor);
}

inline
CBytesPerSecond
CBytesPerSecond::operator % (const CBytesPerSecond& bytespersecond) const
{
	return CBytesPerSecond(((sint32) m_fBytesPerSecond) % ((sint32) bytespersecond.m_fBytesPerSecond));
}

//
//	Special arithmetics
//

inline
CBytesPerSecond
operator / (const CBytes& bytes, const CMilliSeconds& milliseconds)
{
	return CBytesPerSecond((bytes.GetValue() * 1000.0) / (float) milliseconds.GetValue());
}

inline
CMilliSeconds
operator / (const CBytes& bytes, const CBytesPerSecond& bytespersecond)
{
	return CMilliSeconds(((float) bytes.GetValue() * 1000.0) / bytespersecond.GetValue());
}

inline
float
operator / (const CBytesPerSecond& bytespersecond1, const CBytesPerSecond& bytespersecond2)
{
	return ((float) bytespersecond1.GetValue() / (float) bytespersecond2.GetValue());
}

inline
CBytes
operator * (const CBytesPerSecond& bytespersecond, const CMilliSeconds& milliseconds)
{
	return CBytes((float) bytespersecond.GetValue() * (float) milliseconds.GetValue() / 1000.0);
}

inline
CBytes
operator * (const CMilliSeconds& milliseconds, const CBytesPerSecond& bytespersecond)
{
	return CBytes((float) milliseconds.GetValue() * (float) bytespersecond.GetValue() / 1000.0);
}

inline
CBytesPerSecond
operator * (const float& multiplier, const CBytesPerSecond& bytespersecond)
{
	return CBytesPerSecond(multiplier * bytespersecond.GetValue());
}

//
//	
//

#include "OtherFunctions.h"

inline
CString
CastItoXBytes(const CBytesPerSecond& bytespersecond, uint32 decimal = 2)
{
	if (bytespersecond >= 0)
		return ::CastItoXBytes((uint32) bytespersecond.GetValue(), false, true, decimal);
	else
		return _T("-") + ::CastItoXBytes((uint32) (-bytespersecond.GetValue()), false, true, decimal);
}
